from aiohttp.abc import AbstractAccessLogger
from aiohttp.web_request import BaseRequest
from aiohttp.web_response import StreamResponse
from . import REDIRECT_CODES

class AccessLogger(AbstractAccessLogger):

    def log(self, request: BaseRequest, response: StreamResponse, time: float):
        level = self.logger.info
        if response.status >= 400:
            level = self.logger.warning
        elif response.status >= 500:
            level = self.logger.error
        extra = dict(
            METHOD=request.method,
            PATH=request.rel_url,
            QUERY=request.query_string,
            PROTOCOL=request.scheme,
            TIME=time,
            STATUS_CODE=response.status_code,
            REMOTE=request.remote,
            LENGTH=response.content_length,
        )
        if request.headers.get("Referer", False):
            extra["REFERER"] = request.headers.get("Referer")
        if request.headers.get("user-agent", False):
            extra["USER_AGENT"] = request.headers.get("user-agent")
        location=""
        if response.status_code in REDIRECT_CODES:
            extra["LOCATION"] = request.headers.get("location")
            location = f" -> {extra["LOCATION"]}"
        level(f"Access({response.status}) {request.method} {request.rel_url}{}", extra=extra)
